@@ -14,7 +14,7 @@ from util import *
 
 from docutils.statemachine import ViewList
 
-from sphinx.ext.autodoc import AutoDirective, Documenter, add_documenter, \
+from sphinx.ext.autodoc import AutoDirective, add_documenter, \
      ModuleLevelDocumenter, FunctionDocumenter, cut_lines, between, ALL
 
 
@@ -97,28 +97,28 @@ def test_parse_name():
     verify('function', 'util.raises', ('util', ['raises'], None, None))
     verify('function', 'util.raises(exc) -> None',
            ('util', ['raises'], 'exc', 'None'))
-    directive.env.autodoc_current_module = 'util'
+    directive.env.temp_data['autodoc:module'] = 'util'
     verify('function', 'raises', ('util', ['raises'], None, None))
-    directive.env.autodoc_current_module = None
-    directive.env.currmodule = 'util'
+    del directive.env.temp_data['autodoc:module']
+    directive.env.temp_data['py:module'] = 'util'
     verify('function', 'raises', ('util', ['raises'], None, None))
     verify('class', 'TestApp', ('util', ['TestApp'], None, None))
 
     # for members
-    directive.env.currmodule = 'foo'
+    directive.env.temp_data['py:module'] = 'foo'
     verify('method', 'util.TestApp.cleanup',
            ('util', ['TestApp', 'cleanup'], None, None))
-    directive.env.currmodule = 'util'
-    directive.env.currclass = 'Foo'
-    directive.env.autodoc_current_class = 'TestApp'
+    directive.env.temp_data['py:module'] = 'util'
+    directive.env.temp_data['py:class'] = 'Foo'
+    directive.env.temp_data['autodoc:class'] = 'TestApp'
     verify('method', 'cleanup', ('util', ['TestApp', 'cleanup'], None, None))
     verify('method', 'TestApp.cleanup',
            ('util', ['TestApp', 'cleanup'], None, None))
 
     # and clean up
-    directive.env.currmodule = None
-    directive.env.currclass = None
-    directive.env.autodoc_current_class = None
+    del directive.env.temp_data['py:module']
+    del directive.env.temp_data['py:class']
+    del directive.env.temp_data['autodoc:class']
 
 
 def test_format_signature():
@@ -169,8 +169,9 @@ def test_format_signature():
     assert formatsig('method', 'H.foo', H.foo1, 'a', None) == '(a)'
     assert formatsig('method', 'H.foo', H.foo2, None, None) == '(b, *c)'
 
-    # test exception handling
-    raises(TypeError, formatsig, 'function', 'int', int, None, None)
+    # test exception handling (exception is caught and args is '')
+    assert formatsig('function', 'int', int, None, None) == ''
+    del _warnings[:]
 
     # test processing by event handler
     assert formatsig('method', 'bar', H.foo1, None, None) == '42'
@@ -270,7 +271,7 @@ def test_docstring_processing():
     app.disconnect(lid)
 
     lid = app.connect('autodoc-process-docstring', between('---', ['function']))
-    def f():
+    def g():
         """
         first line
         ---
@@ -278,9 +279,21 @@ def test_docstring_processing():
         ---
         third line
         """
-    assert process('function', 'f', f) == ['second line', '']
+    assert process('function', 'g', g) == ['second line', '']
     app.disconnect(lid)
 
+    lid = app.connect('autodoc-process-docstring', between('---', ['function'],
+                                                           exclude=True))
+    def h():
+        """
+        first line
+        ---
+        second line
+        ---
+        third line
+        """
+    assert process('function', 'h', h) == ['first line', 'third line', '']
+    app.disconnect(lid)
 
 def test_new_documenter():
     class MyDocumenter(ModuleLevelDocumenter):
@@ -306,7 +319,7 @@ def test_new_documenter():
         del directive.result[:]
 
     options.members = ['integer']
-    assert_result_contains('.. data:: integer', 'module', 'test_autodoc')
+    assert_result_contains('.. py:data:: integer', 'module', 'test_autodoc')
 
 
 def test_generate():
@@ -340,6 +353,26 @@ def test_generate():
         assert item in directive.result
         del directive.result[:]
 
+    def assert_order(items, objtype, name, member_order, **kw):
+        inst = AutoDirective._registry[objtype](directive, name)
+        inst.options.member_order = member_order
+        inst.generate(**kw)
+        assert len(_warnings) == 0, _warnings
+        items = list(reversed(items))
+        lineiter = iter(directive.result)
+        #for line in directive.result:
+        #    if line.strip():
+        #        print repr(line)
+        while items:
+            item = items.pop()
+            for line in lineiter:
+                if line == item:
+                    break
+            else:  # ran out of items!
+                assert False, 'item %r not found in result or not in the ' \
+                       ' correct order' % item
+        del directive.result[:]
+
     options.members = []
 
     # no module found?
@@ -353,7 +386,7 @@ def test_generate():
                  'function', 'util.foobar', more_content=None)
 
     # test auto and given content mixing
-    directive.env.currmodule = 'test_autodoc'
+    directive.env.temp_data['py:module'] = 'test_autodoc'
     assert_result_contains('   Function.', 'method', 'Class.meth')
     add_content = ViewList()
     add_content.append('Content.', '', 0)
@@ -382,7 +415,10 @@ def test_generate():
                    ('attribute', 'test_autodoc.Class.descr'),
                    ('attribute', 'test_autodoc.Class.attr'),
                    ('attribute', 'test_autodoc.Class.docattr'),
-                   ('attribute', 'test_autodoc.Class.udocattr')])
+                   ('attribute', 'test_autodoc.Class.udocattr'),
+                   ('attribute', 'test_autodoc.Class.inst_attr_comment'),
+                   ('attribute', 'test_autodoc.Class.inst_attr_string')
+                   ])
     options.members = ALL
     assert_processes(should, 'class', 'Class')
     options.undoc_members = True
@@ -394,7 +430,8 @@ def test_generate():
 
     options.members = []
     # test module flags
-    assert_result_contains('.. module:: test_autodoc', 'module', 'test_autodoc')
+    assert_result_contains('.. py:module:: test_autodoc',
+                           'module', 'test_autodoc')
     options.synopsis = 'Synopsis'
     assert_result_contains('   :synopsis: Synopsis', 'module', 'test_autodoc')
     options.deprecated = True
@@ -403,9 +440,9 @@ def test_generate():
     assert_result_contains('   :platform: Platform', 'module', 'test_autodoc')
     # test if __all__ is respected for modules
     options.members = ALL
-    assert_result_contains('.. class:: Class', 'module', 'test_autodoc')
+    assert_result_contains('.. py:class:: Class(arg)', 'module', 'test_autodoc')
     try:
-        assert_result_contains('.. exception:: CustomEx',
+        assert_result_contains('.. py:exception:: CustomEx',
                                'module', 'test_autodoc')
     except AssertionError:
         pass
@@ -419,7 +456,7 @@ def test_generate():
     assert_result_contains('   :noindex:', 'class', 'Base')
 
     # okay, now let's get serious about mixing Python and C signature stuff
-    assert_result_contains('.. class:: CustomDict', 'class', 'CustomDict',
+    assert_result_contains('.. py:class:: CustomDict', 'class', 'CustomDict',
                            all_members=True)
 
     # test inner class handling
@@ -433,10 +470,26 @@ def test_generate():
                            'attribute', 'test_autodoc.Class.descr')
 
     # test generation for C modules (which have no source file)
-    directive.env.currmodule = 'time'
+    directive.env.temp_data['py:module'] = 'time'
     assert_processes([('function', 'time.asctime')], 'function', 'asctime')
     assert_processes([('function', 'time.asctime')], 'function', 'asctime')
 
+    # test autodoc_member_order == 'source'
+    directive.env.temp_data['py:module'] = 'test_autodoc'
+    assert_order(['.. py:class:: Class(arg)',
+                  '   .. py:attribute:: Class.descr',
+                  '   .. py:method:: Class.meth()',
+                  '   .. py:method:: Class.undocmeth()',
+                  '   .. py:attribute:: Class.attr',
+                  '   .. py:attribute:: Class.prop',
+                  '   .. py:attribute:: Class.docattr',
+                  '   .. py:attribute:: Class.udocattr',
+                  '   .. py:attribute:: Class.inst_attr_comment',
+                  '   .. py:attribute:: Class.inst_attr_string',
+                  '   .. py:method:: Class.inheritedmeth()',
+                  ],
+                 'class', 'Class', member_order='bysource', all_members=True)
+
 
 # --- generate fodder ------------
 
@@ -499,6 +552,13 @@ class Class(Base):
     udocattr = 'quux'
     u"""should be documented as well - süß"""
 
+    def __init__(self, arg):
+        #: a documented instance attribute
+        self.inst_attr_comment = None
+        self.inst_attr_string = None
+        """a documented instance attribute"""
+
+
 class CustomDict(dict):
     """Docstring."""
 
